#include <iostream>
#include <stdlib.h>
#include <time.h>
#include "world.hh"

// overload + 
Rgb operator+(Rgb a, Rgb b)
{
  Rgb temp;
  temp.r = a.r + b.r;
  temp.g = a.g + b.g;
  temp.b = a.b + b.b;
  return temp;
}

World::World(short x, short y) : xsize(x), ysize(y)
{
  data = new guchar[(x + 2) * (y + 2) * 3];  // * 3 for RGB // +2 for border
}

World::~World()
{
  delete []data;
}

inline bool World::inbounds(short x, short y)
{
  if (x > xsize || y > ysize) {
    cout << "Value out of bounds!" << endl;
    return FALSE;
  }
  return TRUE;
}

inline Rgb World::val(short x, short y)
{
  /*
  if (!(inbounds(x,y)))
    return 0;
  */
  Rgb rgb;
  rgb.r = data[3 * (x + y * (xsize + 2)) + 0];
  rgb.g = data[3 * (x + y * (xsize + 2)) + 1];
  rgb.b = data[3 * (x + y * (xsize + 2)) + 2];
  return rgb;
}

inline bool World::val(short x, short y, Rgb rgb)
{
  if (!(inbounds(x,y)))
    return FALSE;
  data[3 * (x + y * (xsize + 2)) + 0] = rgb.r;
  data[3 * (x + y * (xsize + 2)) + 1] = rgb.g;
  data[3 * (x + y * (xsize + 2)) + 2] = rgb.b;
  return TRUE;
}

inline bool World::val(short x, short y, char r, char g, char b)
{
  if (!(inbounds(x,y)))
    return FALSE;
  data[3 * (x + y * (xsize + 2)) + 0] = r;
  data[3 * (x + y * (xsize + 2)) + 1] = g;
  data[3 * (x + y * (xsize + 2)) + 2] = b;
  return TRUE;
}

bool World::fill(short x1, short y1, short x2, short y2, Rgb rgb)
{
  if (!(inbounds(x1,y1) && inbounds(x2,y2)))
    return FALSE;
  x1++; x2++; y1++; y2++;
  for (short y = y1; y < y2; y++)
    for (short x = x1; x < x2; x++) {
      data[3 * (x + y * (xsize + 2)) + 0] = rgb.r;
      data[3 * (x + y * (xsize + 2)) + 1] = rgb.g;
      data[3 * (x + y * (xsize + 2)) + 2] = rgb.b;
    }
}

bool World::fill(short x1, short y1, short x2, short y2, char r, char g, char b)
{
  if (!(inbounds(x1,y1) && inbounds(x2,y2)))
    return FALSE;
  x1++; x2++; y1++; y2++;
  for (short y = y1; y < y2; y++)
    for (short x = x1; x < x2; x++) {
      data[3 * (x + y * (xsize + 2)) + 0] = r;
      data[3 * (x + y * (xsize + 2)) + 1] = g;
      data[3 * (x + y * (xsize + 2)) + 2] = b;
    }
}

bool World::fill_rand(short x1, short y1, short x2, short y2, short chance)
{
  if (!(inbounds(x1,y1) && inbounds(x2,y2)))
    return FALSE;
  x1++; x2++; y1++; y2++;
  srand(time(NULL));
  for (short y = y1; y < y2; y++)
    for (short x = x1; x < x2; x++) {
      if (rand() % chance == 0) {
	data[3 * (x + y * (xsize + 2)) + 0] = rand() % 256;
	data[3 * (x + y * (xsize + 2)) + 1] = rand() % 256;
	data[3 * (x + y * (xsize + 2)) + 2] = rand() % 256;
      }
    }
}

void World::cycle()
{
  Rgb rgb;
  for(int y = 1; y <= ysize; y++)
    for(int x = 1; x <= xsize; x++) {
      rgb = val(x - 1,y) + val(x + 1,y) + val(x,y + 1) + val(x,y - 1);
      val(x,y,rgb);
      /*
	cout << "World::cycle() says: " 
	<< (int)rgb.r << "," 
	<< (int)rgb.g << "," 
	<< (int)rgb.b << endl;
      */
    }
}
